@@ -84,6 +84,31 @@ VALID_STORE_RESULTS = {
 }
 
 
+# Some of the values returned by the "stats" command
+# need mapping into native Python types
+STAT_TYPES = {
+    # General stats
+    'version': str,
+    'rusage_user': lambda value: float(value.replace(':', '.')),
+    'rusage_system': lambda value: float(value.replace(':', '.')),
+    'hash_is_expanding': lambda value: int(value) != 0,
+    'slab_reassign_running': lambda value: int(value) != 0,
+
+    # Settings stats
+    'inter': str,
+    'evictions': lambda value: value == 'on',
+    'growth_factor': float,
+    'stat_key_prefix': str,
+    'umask': lambda value: int(value, 8),
+    'detail_enabled': lambda value: int(value) != 0,
+    'cas_enabled': lambda value: int(value) != 0,
+    'auth_enabled_sasl': lambda value: value == 'yes',
+    'maxconns_fast': lambda value: int(value) != 0,
+    'slab_reassign': lambda value: int(value) != 0,
+    'slab_automove': lambda value: int(value) != 0,
+}
+
+
 class MemcacheError(Exception):
     "Base exception class"
     pass
@@ -171,7 +196,7 @@ class Client(object):
               return value
           if flags == 2:
               return json.loads(value)
-          raise Exception("Unknown flags for value: {}".format(flags))
+          raise Exception("Unknown flags for value: {1}".format(flags))
 
     Error Handling:
     ---------------
@@ -454,7 +479,7 @@ class Client(object):
           If noreply is True, always returns True. Otherwise returns True if
           the key was deleted, and False if it wasn't found.
         """
-        cmd = 'delete {}{}\r\n'.format(key, ' noreply' if noreply else '')
+        cmd = 'delete {0}{1}\r\n'.format(key, ' noreply' if noreply else '')
         result = self._misc_cmd(cmd, 'delete', noreply)
         if noreply:
             return True
@@ -496,7 +521,7 @@ class Client(object):
           If noreply is True, always returns None. Otherwise returns the new
           value of the key, or None if the key wasn't found.
         """
-        cmd = "incr {} {}{}\r\n".format(
+        cmd = "incr {0} {1}{2}\r\n".format(
             key,
             str(value),
             ' noreply' if noreply else '')
@@ -520,7 +545,7 @@ class Client(object):
           If noreply is True, always returns None. Otherwise returns the new
           value of the key, or None if the key wasn't found.
         """
-        cmd = "decr {} {}{}\r\n".format(
+        cmd = "decr {0} {1}{2}\r\n".format(
             key,
             str(value),
             ' noreply' if noreply else '')
@@ -545,7 +570,7 @@ class Client(object):
           True if the expiration time was updated, False if the key wasn't
           found.
         """
-        cmd = "touch {} {}{}\r\n".format(
+        cmd = "touch {0} {1}{2}\r\n".format(
             key,
             expire,
             ' noreply' if noreply else '')
@@ -554,9 +579,31 @@ class Client(object):
             return True
         return result == 'TOUCHED'
 
-    def stats(self):
-        # TODO(charles)
-        pass
+    def stats(self, *args):
+        """
+        The memcached "stats" command.
+
+        The returned keys depend on what the "stats" command returns.
+        A best effort is made to convert values to appropriate Python
+        types, defaulting to strings when a conversion cannot be made.
+
+        Args:
+          *arg: extra string arguments to the "stats" command. See the
+                memcached protocol documentation for more information.
+
+        Returns:
+          A dict of the returned stats.
+        """
+        result = self._fetch_cmd('stats', args, False)
+
+        for key, value in result.iteritems():
+            converter = STAT_TYPES.get(key, int)
+            try:
+                result[key] = converter(value)
+            except Exception:
+                pass
+
+        return result
 
     def flush_all(self, delay=0, noreply=True):
         """
@@ -570,7 +617,7 @@ class Client(object):
         Returns:
           True.
         """
-        cmd = "flush_all {}{}\r\n".format(delay, ' noreply' if noreply else '')
+        cmd = "flush_all {0}{1}\r\n".format(delay, ' noreply' if noreply else '')
         result = self._misc_cmd(cmd, 'flush_all', noreply)
         if noreply:
             return True
@@ -614,7 +661,7 @@ class Client(object):
         except UnicodeEncodeError as e:
             raise MemcacheIllegalInputError(str(e))
 
-        cmd = '{} {}\r\n'.format(name, ' '.join(key_strs))
+        cmd = '{0} {1}\r\n'.format(name, ' '.join(key_strs))
 
         try:
             self.sock.sendall(cmd)
@@ -643,6 +690,9 @@ class Client(object):
                         result[key] = (value, cas)
                     else:
                         result[key] = value
+                elif name == 'stats' and line.startswith('STAT'):
+                    _, key, value = line.split()
+                    result[key] = value
                 else:
                     raise MemcacheUnknownError(line[:32])
         except Exception:
@@ -673,15 +723,15 @@ class Client(object):
             raise MemcacheIllegalInputError(str(e))
 
         if cas is not None and noreply:
-            extra = ' {} noreply'.format(cas)
+            extra = ' {0} noreply'.format(cas)
         elif cas is not None and not noreply:
-            extra = ' {}'.format(cas)
+            extra = ' {0}'.format(cas)
         elif cas is None and noreply:
             extra = ' noreply'
         else:
             extra = ''
 
-        cmd = '{} {} {} {} {}{}\r\n{}\r\n'.format(
+        cmd = '{0} {1} {2} {3} {4}{5}\r\n{6}\r\n'.format(
             name, key, flags, expire, len(data), extra, data)
 
         try:
